@@ -5,11 +5,11 @@
 import os
 import sys
 import argparse
-import logging
 
-from doorstop.cli import utilities, commands
-from doorstop.gui.main import _run as gui
 from doorstop import common
+from doorstop.cli import utilities, commands
+
+log = common.logger(__name__)
 
 
 def main(args=None):  # pylint: disable=R0915
@@ -17,13 +17,27 @@ def main(args=None):  # pylint: disable=R0915
     from doorstop import CLI, VERSION, DESCRIPTION
 
     # Shared options
+    project = argparse.ArgumentParser(add_help=False)
+    project.add_argument('-j', '--project', metavar='PATH',
+                         help="path to the root of the project")
+    project.add_argument('--no-cache', action='store_true',
+                         help=argparse.SUPPRESS)
+    server = argparse.ArgumentParser(add_help=False)
+    server.add_argument('--server', metavar='HOST',
+                        help="IP address or hostname for a running server")
+    server.add_argument('--port', metavar='NUM', type=int,
+                        help="use a custom port for the server")
+    server.add_argument('-f', '--force', action='store_true',
+                        help="perform the action without the server")
     debug = argparse.ArgumentParser(add_help=False)
-    debug.add_argument('-j', '--project', metavar='PATH',
-                       help="path to the root of the project")
     debug.add_argument('-V', '--version', action='version', version=VERSION)
-    debug.add_argument('-v', '--verbose', action='count', default=0,
+    group = debug.add_mutually_exclusive_group()
+    group.add_argument('-v', '--verbose', action='count', default=0,
                        help="enable verbose logging")
-    shared = {'formatter_class': common.HelpFormatter, 'parents': [debug]}
+    group.add_argument('-q', '--quiet', action='store_const', const=-1,
+                       dest='verbose', help="only display errors and prompts")
+    shared = {'formatter_class': common.HelpFormatter,
+              'parents': [project, server, debug]}
 
     # Build main parser
     parser = argparse.ArgumentParser(prog=CLI, description=DESCRIPTION,
@@ -38,8 +52,14 @@ def main(args=None):  # pylint: disable=R0915
                         help="do not validate external file references")
     parser.add_argument('-C', '--no-child-check', action='store_true',
                         help="do not validate child (reverse) links")
-    parser.add_argument('-g', '--gui', action='store_true',
-                        help="launch the graphical user interface")
+    parser.add_argument('-S', '--no-suspect-check', action='store_true',
+                        help="do not check for suspect links")
+    parser.add_argument('-W', '--no-review-check', action='store_true',
+                        help="do not check item review status")
+    parser.add_argument('-w', '--warn-all', action='store_true',
+                        help="display all info-level issues as warnings")
+    parser.add_argument('-e', '--error-all', action='store_true',
+                        help="display all warning-level issues as errors")
 
     # Build sub-parsers
     subs = parser.add_subparsers(help="", dest='command', metavar="<command>")
@@ -47,9 +67,12 @@ def main(args=None):  # pylint: disable=R0915
     _delete(subs, shared)
     _add(subs, shared)
     _remove(subs, shared)
+    _edit(subs, shared)
+    _reorder(subs, shared)
     _link(subs, shared)
     _unlink(subs, shared)
-    _edit(subs, shared)
+    _clear(subs, shared)
+    _review(subs, shared)
     _import(subs, shared)
     _export(subs, shared)
     _publish(subs, shared)
@@ -64,20 +87,16 @@ def main(args=None):  # pylint: disable=R0915
     utilities.configure_settings(args)
 
     # Run the program
-    if args.gui:
-        logging.debug("launching GUI...")
-        function = gui
-    else:
-        function = commands.get(args.command)
+    function = commands.get(args.command)
     try:
         success = function(args, os.getcwd(), parser.error)
     except KeyboardInterrupt:
-        logging.debug("command cancelled")
+        log.debug("command cancelled")
         success = False
     if success:
-        logging.debug("command succeeded")
+        log.debug("command succeeded")
     else:
-        logging.debug("command failed")
+        log.debug("command failed")
         sys.exit(1)
 
 
@@ -86,10 +105,10 @@ def _create(subs, shared):
     info = "create a new document directory"
     sub = subs.add_parser('create', description=info.capitalize() + '.',
                           help=info, **shared)
-    sub.add_argument('prefix', help="document prefix for new item IDs")
+    sub.add_argument('prefix', help="document prefix for new item UIDs")
     sub.add_argument('path', help="path to a directory for item files")
-    sub.add_argument('-p', '--parent', help="prefix for parent item IDS")
-    sub.add_argument('-d', '--digits', help="number of digits in item IDs")
+    sub.add_argument('-p', '--parent', help="prefix of parent document")
+    sub.add_argument('-d', '--digits', help="number of digits in item UIDs")
 
 
 def _delete(subs, shared):
@@ -108,6 +127,8 @@ def _add(subs, shared):
     sub.add_argument('prefix',
                      help="document prefix for the new item")
     sub.add_argument('-l', '--level', help="desired item level (e.g. 1.2.3)")
+    sub.add_argument('-c', '--count', default=1, type=utilities.positive_int,
+                     help="number of items to create (default: 1)")
 
 
 def _remove(subs, shared):
@@ -115,30 +136,7 @@ def _remove(subs, shared):
     info = "remove an item file from a document directory"
     sub = subs.add_parser('remove', description=info.capitalize() + '.',
                           help=info, **shared)
-    sub.add_argument('id', metavar='ID',
-                     help="item ID to remove from its document")
-
-
-def _link(subs, shared):
-    """Configure the `doorstop link` subparser."""
-    info = "add a new link between two items"
-    sub = subs.add_parser('link', description=info.capitalize() + '.',
-                          help=info, **shared)
-    sub.add_argument('child',
-                     help="child item ID to link to the parent")
-    sub.add_argument('parent',
-                     help="parent item ID to link from the child")
-
-
-def _unlink(subs, shared):
-    """Configure the `doorstop unlink` subparser."""
-    info = "remove a link between two items"
-    sub = subs.add_parser('unlink', description=info.capitalize() + '.',
-                          help=info, **shared)
-    sub.add_argument('child',
-                     help="child item ID to unlink from parent")
-    sub.add_argument('parent',
-                     help="parent item ID child is linked to")
+    sub.add_argument('uid', help="item UID to remove from its document")
 
 
 def _edit(subs, shared):
@@ -147,10 +145,10 @@ def _edit(subs, shared):
     sub = subs.add_parser('edit', description=info.capitalize() + '.',
                           help=info, **shared)
     sub.add_argument('label',
-                     help="item ID or document prefix to open for editing")
+                     help="item UID or document prefix to open for editing")
     group = sub.add_mutually_exclusive_group()
     group.add_argument('-i', '--item', action='store_true',
-                       help="indicates the 'label' is an item ID")
+                       help="indicates the 'label' is an item UID")
     group.add_argument('-d', '--document', action='store_true',
                        help="indicates the 'label' is a document prefix")
     group = sub.add_mutually_exclusive_group()
@@ -166,6 +164,69 @@ def _edit(subs, shared):
                      help="text editor to open the document item")
 
 
+def _reorder(subs, shared):
+    """Configure the `doorstop reorder` subparser."""
+    info = "organize the outline structure of a document"
+    sub = subs.add_parser('reorder', description=info.capitalize() + '.',
+                          help=info, **shared)
+    sub.add_argument('prefix', help="prefix of document to reorder")
+    group = sub.add_mutually_exclusive_group()
+    group.add_argument('-a', '--auto', action='store_true',
+                       help="only perform automatic item reordering")
+    group.add_argument('-m', '--manual', action='store_true',
+                       help="do not automatically reorder the items")
+    sub.add_argument('-T', '--tool', metavar='PROGRAM',
+                     help="text editor to open the document index")
+
+
+def _link(subs, shared):
+    """Configure the `doorstop link` subparser."""
+    info = "add a new link between two items"
+    sub = subs.add_parser('link', description=info.capitalize() + '.',
+                          help=info, **shared)
+    sub.add_argument('child',
+                     help="child item UID to link to the parent")
+    sub.add_argument('parent',
+                     help="parent item UID to link from the child")
+
+
+def _unlink(subs, shared):
+    """Configure the `doorstop unlink` subparser."""
+    info = "remove a link between two items"
+    sub = subs.add_parser('unlink', description=info.capitalize() + '.',
+                          help=info, **shared)
+    sub.add_argument('child',
+                     help="child item UID to unlink from parent")
+    sub.add_argument('parent',
+                     help="parent item UID child is linked to")
+
+
+def _clear(subs, shared):
+    """Configure the `doorstop clear` subparser."""
+    info = "absolve items of their suspect link status"
+    sub = subs.add_parser('clear', description=info.capitalize() + '.',
+                          help=info, **shared)
+    sub.add_argument('label', help="item UID, document prefix, or 'all'")
+    group = sub.add_mutually_exclusive_group()
+    group.add_argument('-i', '--item', action='store_true',
+                       help="indicates the 'label' is an item UID")
+    group.add_argument('-d', '--document', action='store_true',
+                       help="indicates the 'label' is a document prefix")
+
+
+def _review(subs, shared):
+    """Configure the `doorstop review` subparser."""
+    info = "absolve items of their unreviewed status"
+    sub = subs.add_parser('review', description=info.capitalize() + '.',
+                          help=info, **shared)
+    sub.add_argument('label', help="item UID, document prefix, or 'all'")
+    group = sub.add_mutually_exclusive_group()
+    group.add_argument('-i', '--item', action='store_true',
+                       help="indicates the 'label' is an item UID")
+    group.add_argument('-d', '--document', action='store_true',
+                       help="indicates the 'label' is a document prefix")
+
+
 def _import(subs, shared):
     """Configure the `doorstop import` subparser."""
     info = "import an existing document or item"
@@ -178,7 +239,7 @@ def _import(subs, shared):
     group.add_argument('-d', '--document', nargs=2, metavar='ARG',
                        help="import an existing document by: PREFIX PATH")
     group.add_argument('-i', '--item', nargs=2, metavar='ARG',
-                       help="import an existing item by: PREFIX ID")
+                       help="import an existing item by: PREFIX UID")
     sub.add_argument('-p', '--parent', metavar='PREFIX',
                      help="parent document prefix for imported document")
     sub.add_argument('-a', '--attrs', metavar='DICT',
@@ -226,7 +287,9 @@ def _publish(subs, shared):
     sub.add_argument('-w', '--width', type=int,
                      help="limit line width on text output")
     sub.add_argument('-C', '--no-child-links', action='store_true',
-                     help="do not include child links in published documents")
+                     help="do not include child links on items")
+    sub.add_argument('-L', '--no-body-levels', action='store_true',
+                     help="do not include levels on non-heading items")
 
 
 if __name__ == '__main__':  # pragma: no cover (manual test)
