@@ -5,10 +5,14 @@ from unittest.mock import patch, Mock, MagicMock
 
 import os
 
+from doorstop import common
 from doorstop.common import DoorstopError
-from doorstop.core.item import Item
+from doorstop.core.types import Text, Stamp
+from doorstop.core.item import Item, UnknownItem
+from doorstop.core.vcs.mockvcs import WorkingCopy
 
-from doorstop.core.test import FILES, EMPTY, EXTERNAL, MockFileObject
+from doorstop.core.test import FILES, EMPTY, EXTERNAL
+from doorstop.core.test import MockItem
 
 
 YAML_DEFAULT = """
@@ -18,18 +22,14 @@ level: 1.0
 links: []
 normative: true
 ref: ''
+reviewed: null
 text: ''
 """.lstrip()
 
 
-class MockItem(MockFileObject, Item):  # pylint: disable=W0223,R0902,R0904
+class TestItem(unittest.TestCase):
 
-    """Mock Item class with stubbed file IO."""
-
-
-class TestItem(unittest.TestCase):  # pylint: disable=R0904
-
-    """Unit tests for the Item class."""  # pylint: disable=C0103,W0212
+    """Unit tests for the Item class."""  # pylint: disable=W0212
 
     def setUp(self):
         path = os.path.join('path', 'to', 'RQ001.yml')
@@ -39,6 +39,11 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         """Verify an item cannot be initialized from an invalid path."""
         self.assertRaises(DoorstopError, Item, 'not/a/path')
 
+    def test_object_references(self):
+        """Verify a standalone item does not have object references."""
+        self.assertIs(None, self.item.document)
+        self.assertIs(None, self.item.tree)
+
     def test_load_empty(self):
         """Verify loading calls read."""
         self.item.load()
@@ -59,17 +64,30 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.item.save()
         self.item._write.assert_called_once_with(YAML_DEFAULT, self.item.path)
 
-    @patch('doorstop.common.VERBOSITY', 2)
+    @patch('doorstop.common.verbosity', 2)
     def test_str(self):
-        """Verify an item can be converted to strings."""
+        """Verify an item can be converted to a string."""
         self.assertEqual("RQ001", str(self.item))
 
-    @patch('doorstop.common.VERBOSITY', 3)
+    @patch('doorstop.common.verbosity', 3)
     def test_str_verbose(self):
-        """Verify an item can be converted to strings in verbose mode."""
+        """Verify an item can be converted to a string (verbose)."""
         text = "RQ001 (@{}{})".format(os.sep, self.item.path)
         self.assertEqual(text, str(self.item))
 
+    def test_hash(self):
+        """Verify items can be hashed."""
+        item1 = MockItem('path/to/fake1.yml')
+        item2 = MockItem('path/to/fake2.yml')
+        item3 = MockItem('path/to/fake2.yml')
+        my_set = set()
+        # Act
+        my_set.add(item1)
+        my_set.add(item2)
+        my_set.add(item3)
+        # Assert
+        self.assertEqual(2, len(my_set))
+
     def test_ne(self):
         """Verify item non-equality is correct."""
         self.assertNotEqual(self.item, None)
@@ -86,10 +104,10 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.assertLess(item2, item3)
         self.assertGreater(item3, item1)
 
-    def test_id(self):
-        """Verify an item's ID can be read but not set."""
-        self.assertEqual('RQ001', self.item.id)
-        self.assertRaises(AttributeError, setattr, self.item, 'id', 'RQ002')
+    def test_uid(self):
+        """Verify an item's UID can be read but not set."""
+        self.assertEqual('RQ001', self.item.uid)
+        self.assertRaises(AttributeError, setattr, self.item, 'uid', 'RQ002')
 
     def test_relpath(self):
         """Verify an item's relative path string can be read but not set."""
@@ -182,6 +200,28 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.assertTrue(self.item.normative)
         self.assertFalse(self.item.heading)
 
+    def test_cleared(self):
+        """Verify an item's suspect link status can be set and read."""
+        mock_item = Mock()
+        mock_item.uid = 'mock_uid'
+        mock_item.stamp = Mock(return_value=Stamp('abc123'))
+        mock_tree = MagicMock()
+        mock_tree.find_item = Mock(return_value=mock_item)
+        self.item.tree = mock_tree
+        self.item.link('mock_uid')
+        self.item.cleared = 1  # updates each stamp
+        self.assertTrue(self.item.cleared)
+        self.item.cleared = 0  # sets each stamp to None
+        self.assertFalse(self.item.cleared)
+
+    def test_reviwed(self):
+        """Verify an item's review status can be set and read."""
+        self.assertFalse(self.item.reviewed)  # not reviewed by default
+        self.item.reviewed = 1  # calls `review()`
+        self.assertTrue(self.item.reviewed)
+        self.item.reviewed = 0  # converted to None
+        self.assertFalse(self.item.reviewed)
+
     def test_text(self):
         """Verify an item's text can be set and read."""
         value = "abc "
@@ -257,8 +297,16 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.assertEqual("abc123", self.item.ref)
 
     def test_extended(self):
-        """Verify an extended attribute can be used."""
+        """Verify an extended attribute (`str`) can be used."""
         self.item.set('ext1', 'foobar')
+        self.assertIn("ext1: foobar\n", self.item._write.call_args[0][0])
+        self.assertEqual('foobar', self.item.get('ext1'))
+        self.assertEqual(['ext1'], self.item.extended)
+
+    def test_extended_text(self):
+        """Verify an extended attribute (`Text`) can be used."""
+        self.item.set('ext1', Text('foobar'))
+        self.assertIn("ext1: foobar\n", self.item._write.call_args[0][0])
         self.assertEqual('foobar', self.item.get('ext1'))
         self.assertEqual(['ext1'], self.item.extended)
 
@@ -284,10 +332,16 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.item.set('text', "extended access")
         self.assertEqual("extended access", self.item.text)
 
-    def test_object_references_standalone(self):
-        """Verify a standalone item does not have object references."""
-        self.assertIs(None, self.item.document)
-        self.assertIs(None, self.item.tree)
+    @patch('doorstop.core.editor.launch')
+    def test_edit(self, mock_launch):
+        """Verify an item can be edited."""
+        self.item.tree = Mock()
+        # Act
+        self.item.edit(tool='mock_editor')
+        # Assert
+        self.item.tree.vcs.lock.assert_called_once_with(self.item.path)
+        self.item.tree.vcs.edit.assert_called_once_with(self.item.path)
+        mock_launch.assert_called_once_with(self.item.path, tool='mock_editor')
 
     def test_link(self):
         """Verify links can be added to an item."""
@@ -332,27 +386,104 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.item.links = links2
         self.assertEqual(links2, self.item.parent_links)
 
+    def test_parent_items(self):
+        """Verify 'parent_items' exists to mirror the child behavior."""
+        mock_tree = Mock()
+        mock_tree.find_item = Mock(return_value='mock_item')
+        self.item.tree = mock_tree
+        self.item.links = ['mock_uid']
+        # Act
+        items = self.item.parent_items
+        # Assert
+        self.assertEqual(['mock_item'], items)
+
+    def test_parent_items_unknown(self):
+        """Verify 'parent_items' can handle unknown items."""
+        mock_tree = Mock()
+        mock_tree.find_item = Mock(side_effect=DoorstopError)
+        self.item.tree = mock_tree
+        self.item.links = ['mock_uid']
+        # Act
+        items = self.item.parent_items
+        # Assert
+        self.assertIsInstance(items[0], UnknownItem)
+
+    def test_parent_documents(self):
+        """Verify 'parent_documents' exists to mirror the child behavior."""
+        mock_tree = Mock()
+        mock_tree.find_document = Mock(return_value='mock_document')
+        self.item.tree = mock_tree
+        self.item.links = ['mock_uid']
+        self.item.document = Mock()
+        self.item.document.prefix = 'mock_prefix'
+        # Act
+        documents = self.item.parent_documents
+        # Assert
+        self.assertEqual(['mock_document'], documents)
+
+    def test_parent_documents_unknown(self):
+        """Verify 'parent_documents' can handle unknown documents."""
+        mock_tree = Mock()
+        mock_tree.find_document = Mock(side_effect=DoorstopError)
+        self.item.tree = mock_tree
+        self.item.links = ['mock_uid']
+        self.item.document = Mock()
+        self.item.document.prefix = 'mock_prefix'
+        # Act
+        documents = self.item.parent_documents
+        # Assert
+        self.assertEqual([], documents)
+
+    def test_parent_documents_no_document(self):
+        """Verify 'parent_documents' is only valid with a document."""
+        self.item.tree = Mock()
+        self.assertIs(None, self.item.parent_documents)
+
+    @patch('doorstop.settings.CACHE_PATHS', False)
     def test_find_ref(self):
         """Verify an item's reference can be found."""
-        self.item.ref = "REF" + "123"  # to avoid matching in this file
-        relpath, line = self.item.find_ref(EXTERNAL)
+        self.item.ref = "REF" "123"  # space to avoid matching in this file
+        self.item.tree = Mock()
+        self.item.tree.vcs = WorkingCopy(EXTERNAL)
+        # Act
+        relpath, line = self.item.find_ref()
+        # Assert
         self.assertEqual('text.txt', os.path.basename(relpath))
         self.assertEqual(3, line)
 
     def test_find_ref_filename(self):
         """Verify an item's reference can also be a filename."""
         self.item.ref = "text.txt"
-        relpath, line = self.item.find_ref(FILES)
+        self.item.tree = Mock()
+        self.item.tree.vcs = WorkingCopy(FILES)
+        self.item.tree.vcs._ignores_cache = ["*published*"]
+        # Act
+        relpath, line = self.item.find_ref()
+        # Assert
         self.assertEqual('text.txt', os.path.basename(relpath))
         self.assertEqual(None, line)
 
     def test_find_ref_error(self):
         """Verify an error occurs when no external reference found."""
-        self.item.ref = "not found".replace(' ', '')  # avoids self match
-        self.assertRaises(DoorstopError, self.item.find_ref, root=EMPTY)
+        self.item.ref = "not" "found"  # space to avoid matching in this file
+        self.item.tree = Mock()
+        self.item.tree.vcs = WorkingCopy(EMPTY)
+        # Act and assert
+        self.assertRaises(DoorstopError, self.item.find_ref)
+
+    def test_find_skip_self(self):
+        """Verify reference searches skip the item's file."""
+        self.item.path = __file__
+        self.item.ref = "148710938710289248"  # random and unique to this file
+        self.item.tree = Mock()
+        self.item.tree.vcs = WorkingCopy(EMPTY)
+        self.item.tree.vcs._path_cache = [(__file__, 'filename', 'relpath')]
+        # Act and assert
+        self.assertRaises(DoorstopError, self.item.find_ref)
 
     def test_find_ref_none(self):
         """Verify nothing returned when no external reference is specified."""
+        self.item.tree = Mock()
         self.assertEqual((None, None), self.item.find_ref())
 
     def test_find_child_objects(self):
@@ -365,7 +496,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         mock_document_c.parent = 'RQ'
 
         mock_item = Mock()
-        mock_item.id = 'TST001'
+        mock_item.uid = 'TST001'
         mock_item.links = ['RQ001']
 
         def mock_iter(self):  # pylint: disable=W0613
@@ -381,7 +512,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.item.link('fake1')
         mock_tree = Mock()
         mock_tree.__iter__ = mock_iter
-        mock_tree.find_item = lambda identifier: Mock(id='fake1')
+        mock_tree.find_item = lambda uid: Mock(uid='fake1')
         self.item.tree = mock_tree
         self.item.document = mock_document_p
 
@@ -411,26 +542,37 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
     @patch('doorstop.core.item.Item', MockItem)
     def test_new(self):
         """Verify items can be created."""
-        MockItem._new.reset_mock()
+        MockItem._create.reset_mock()
         item = MockItem.new(None, None,
                             EMPTY, FILES, 'TEST00042',
                             level=(1, 2, 3))
         path = os.path.join(EMPTY, 'TEST00042.yml')
         self.assertEqual(path, item.path)
         self.assertEqual((1, 2, 3), item.level)
-        MockItem._new.assert_called_once_with(path, name='item')
+        MockItem._create.assert_called_once_with(path, name='item')
+
+    @patch('doorstop.core.item.Item', MockItem)
+    def test_new_cache(self):
+        """Verify new items are cached."""
+        mock_tree = Mock()
+        mock_tree._item_cache = {}
+        item = MockItem.new(mock_tree, None,
+                            EMPTY, FILES, 'TEST00042',
+                            level=(1, 2, 3))
+        self.assertEqual(item, mock_tree._item_cache[item.uid])
+        mock_tree.vcs.add.assert_called_once_with(item.path)
 
     @patch('doorstop.core.item.Item', MockItem)
     def test_new_special(self):
         """Verify items can be created with a specially named prefix."""
-        MockItem._new.reset_mock()
+        MockItem._create.reset_mock()
         item = MockItem.new(None, None,
                             EMPTY, FILES, 'VSM.HLR_01-002-042',
                             level=(1, 0))
         path = os.path.join(EMPTY, 'VSM.HLR_01-002-042.yml')
         self.assertEqual(path, item.path)
         self.assertEqual((1,), item.level)
-        MockItem._new.assert_called_once_with(path, name='item')
+        MockItem._create.assert_called_once_with(path, name='item')
 
     def test_new_existing(self):
         """Verify an exception is raised if the item already exists."""
@@ -452,6 +594,24 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
                    Mock(side_effect=DoorstopError)):
             self.assertTrue(self.item.validate())
 
+    def test_validate_reviewed(self):
+        """Verify that checking a reviewed item updates the stamp."""
+        self.item._data['reviewed'] = True
+        self.assertTrue(self.item.validate())
+        stamp = 'c6a87755b8756b61731c704c6a7be4a2'
+        self.assertEqual(stamp, self.item._data['reviewed'])
+
+    def test_validate_cleared(self):
+        """Verify that checking a cleared link updates the stamp."""
+        mock_item = Mock()
+        mock_item.stamp = Mock(return_value=Stamp('abc123'))
+        mock_tree = MagicMock()
+        mock_tree.find_item = Mock(return_value=mock_item)
+        self.item.tree = mock_tree
+        self.item.links = [{'mock_uid': True}]
+        self.assertTrue(self.item.validate())
+        self.assertEqual('abc123', self.item.links[0].stamp)  # pylint: disable=E1101
+
     def test_validate_nonnormative_with_links(self):
         """Verify a non-normative item with links can be checked."""
         self.item.normative = False
@@ -493,8 +653,8 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.item.document = mock_document
         self.assertTrue(self.item.validate())
 
-    def test_validate_document_with_bad_link_IDs(self):
-        """Verify an item can be checked against a document w/ bad link IDs."""
+    def test_validate_document_with_bad_link_uids(self):
+        """Verify an item can be checked against a document w/ bad links."""
         self.item.link('invalid')
         mock_document = Mock()
         mock_document.parent = 'fake'
@@ -512,7 +672,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
             def mock_iter2(self):  # pylint: disable=W0613
                 """Mock Document.__iter__ to yield a mock Item."""
                 mock_item = Mock()
-                mock_item.id = 'TST001'
+                mock_item.uid = 'TST001'
                 mock_item.links = ['RQ001']
                 yield mock_item
 
@@ -523,7 +683,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
 
         mock_tree = Mock()
         mock_tree.__iter__ = mock_iter
-        mock_tree.find_item = lambda identifier: Mock(id='fake1')
+        mock_tree.find_item = lambda uid: Mock(uid='fake1')
 
         self.item.tree = mock_tree
 
@@ -548,7 +708,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
             return _iter
 
         mock_item = Mock()
-        mock_item.links = [self.item.id]
+        mock_item.links = [self.item.uid]
 
         mock_document = Mock()
         mock_document.parent = 'BOTH'
@@ -574,7 +734,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
             def mock_iter2(self):  # pylint: disable=W0613
                 """Mock Document.__iter__ to yield a mock Item."""
                 mock_item = Mock()
-                mock_item.id = 'TST001'
+                mock_item.uid = 'TST001'
                 mock_item.links = []
                 yield mock_item
 
@@ -588,7 +748,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
 
         mock_tree = Mock()
         mock_tree.__iter__ = mock_iter
-        mock_tree.find_item = lambda identifier: Mock(id='fake1')
+        mock_tree.find_item = lambda uid: Mock(uid='fake1')
 
         self.item.document = mock_document
         self.item.tree = mock_tree
@@ -600,40 +760,139 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         """Verify an item's issues convenience property can be accessed."""
         self.assertEqual(0, len(self.item.issues))
 
-    @patch('os.remove')
-    def test_delete(self, mock_remove):
+    def test_stamp(self):
+        """Verify an item's contents can be stamped."""
+        stamp = 'c6a87755b8756b61731c704c6a7be4a2'
+        self.assertEqual(stamp, self.item.stamp())
+
+    def test_stamp_links(self):
+        """Verify an item's contents can be stamped."""
+        self.item.link('mock_link')
+        stamp = '1020719292bbdc4090bd236cf41cd104'
+        self.assertEqual(stamp, self.item.stamp(links=True))
+
+    def test_clear(self):
+        """Verify an item's links can be cleared as suspect."""
+        mock_item = Mock()
+        mock_item.uid = 'mock_uid'
+        mock_item.stamp = Mock(return_value=Stamp('abc123'))
+        mock_tree = MagicMock()
+        mock_tree.find_item = Mock(return_value=mock_item)
+        self.item.tree = mock_tree
+        self.item.link('mock_uid')
+        self.assertFalse(self.item.cleared)
+        self.assertEqual(None, self.item.links[0].stamp)  # pylint: disable=E1101
+        # Act
+        self.item.clear()
+        # Assert
+        self.assertTrue(self.item.cleared)
+        self.assertEqual('abc123', self.item.links[0].stamp)  # pylint: disable=E1101
+
+    def test_review(self):
+        """Verify an item can be marked as reviewed."""
+        self.item.reviewed = False
+        self.item.review()
+        self.assertTrue(self.item.reviewed)
+
+    @patch('doorstop.common.delete')
+    def test_delete(self, mock_delete):
         """Verify an item can be deleted."""
         self.item.delete()
-        mock_remove.assert_called_once_with(self.item.path)
+        mock_delete.assert_called_once_with(self.item.path)
         self.item.delete()  # ensure a second delete is ignored
 
+    @patch('doorstop.common.delete', Mock())
+    def test_delete_cache(self):
+        """Verify an item is expunged after delete."""
+        self.item.tree = Mock()
+        self.item.tree._item_cache = {self.item.uid: self.item}
+        self.item.delete()
+        self.item.tree.vcs.delete.assert_called_once_with(self.item.path)
+        self.assertIs(None, self.item.tree._item_cache[self.item.uid])
 
-class TestFormatting(unittest.TestCase):  # pylint: disable=R0904
 
-    """Unit tests for text formatting in Items."""  # pylint: disable=C0103
+class TestFormatting(unittest.TestCase):
+
+    """Unit tests for text formatting in Items."""
 
     ITEM = os.path.join(FILES, 'REQ001.yml')
 
     def setUp(self):
-        with open(self.ITEM, 'r') as infile:
-            self.backup = infile.read()
+        self.backup = common.read_text(self.ITEM)
 
     def tearDown(self):
-        with open(self.ITEM, 'w') as outfile:
-            outfile.write(self.backup)
+        common.write_text(self.backup, self.ITEM)
 
     def test_load_save(self):
         """Verify text formatting is preserved."""
         item = Item(self.ITEM)
         item.load()
         item.save()
-        with open(self.ITEM, 'r') as infile:
-            text = infile.read()
-            self.assertEqual(self.backup, text)
+        text = common.read_text(self.ITEM)
+        self.assertEqual(self.backup, text)
 
 
-class TestModule(unittest.TestCase):  # pylint: disable=R0904
+class TestUnknownItem(unittest.TestCase):
 
-    """Unit tests for the doorstop.core.item module."""  # pylint: disable=C0103
+    """Unit tests for the UnknownItem class."""  # pylint: disable= W0212
 
-    pass
+    def setUp(self):
+        self.item = UnknownItem('RQ001')
+
+    @patch('doorstop.common.verbosity', 2)
+    def test_str(self):
+        """Verify an unknown item can be converted to a string."""
+        self.assertEqual("RQ001", str(self.item))
+
+    @patch('doorstop.common.verbosity', 3)
+    def test_str_verbose(self):
+        """Verify an unknown item can be converted to a string (verbose)."""
+        text = "RQ001 (@{}{})".format(os.sep, '???')
+        self.assertEqual(text, str(self.item))
+
+    def test_uid(self):
+        """Verify an unknown item's UID can be read but not set."""
+        self.assertEqual('RQ001', self.item.uid)
+        self.assertRaises(AttributeError, setattr, self.item, 'uid', 'RQ002')
+
+    def test_relpath(self):
+        """Verify an item's relative path string can be read but not set."""
+        text = "@{}{}".format(os.sep, '???')
+        self.assertEqual(text, self.item.relpath)
+        self.assertRaises(AttributeError, setattr, self.item, 'relpath', '.')
+
+    def test_prefix(self):
+        """Verify an item's prefix can be read but not set."""
+        self.assertEqual('RQ', self.item.prefix)
+        self.assertRaises(AttributeError, setattr, self.item, 'prefix', 'REQ')
+
+    def test_number(self):
+        """Verify an item's number can be read but not set."""
+        self.assertEqual(1, self.item.number)
+        self.assertRaises(AttributeError, setattr, self.item, 'number', 2)
+
+    @patch('doorstop.core.item.log.debug')
+    def test_attributes(self, mock_warning):
+        """Verify all other `Item` attributes raise an exception."""
+        self.assertRaises(AttributeError, getattr, self.item, 'path')
+        self.assertRaises(AttributeError, getattr, self.item, 'text')
+        self.assertRaises(AttributeError, getattr, self.item, 'get_issues')
+        self.assertRaises(AttributeError, getattr, self.item, 'delete')
+        self.assertRaises(AttributeError, getattr, self.item, 'not_on_item')
+        self.assertEqual(3, mock_warning.call_count)
+
+    @patch('doorstop.core.item.log.debug')
+    def test_attributes_with_spec(self, mock_warning):
+        """Verify all other `Item` attributes raise an exception."""
+        spec = Item(os.path.join(FILES, 'REQ001.yml'))
+        self.item = UnknownItem(self.item.uid, spec=spec)
+        self.assertRaises(AttributeError, getattr, self.item, 'path')
+        self.assertRaises(AttributeError, getattr, self.item, 'text')
+        self.assertRaises(AttributeError, getattr, self.item, 'get_issues')
+        self.assertRaises(AttributeError, getattr, self.item, 'delete')
+        self.assertRaises(AttributeError, getattr, self.item, 'not_on_item')
+        self.assertEqual(4, mock_warning.call_count)
+
+    def test_stamp(self):
+        """Verify an unknown item has no stamp."""
+        self.assertEqual(Stamp(None), self.item.stamp())
