@@ -10,32 +10,36 @@ try:  # pragma: no cover (manual test)
     from tkinter import font, filedialog
 except ImportError as _exc:  # pragma: no cover (manual test)
     sys.stderr.write("WARNING: {}\n".format(_exc))
-    tk = Mock()  # pylint: disable=C0103
-    ttk = Mock()  # pylint: disable=C0103
+    tk = Mock()
+    ttk = Mock()
 import os
 import argparse
 import functools
 from itertools import chain
 import logging
 
+from doorstop import common
 from doorstop.common import HelpFormatter, WarningFormatter, DoorstopError
 from doorstop.core import vcs
 from doorstop.core import builder
 from doorstop import settings
 
+log = common.logger(__name__)
+
 
 def main(args=None):
     """Process command-line arguments and run the program."""
     from doorstop import GUI, VERSION
 
-    # Main parser
+    # Shared options
     debug = argparse.ArgumentParser(add_help=False)
     debug.add_argument('-V', '--version', action='version', version=VERSION)
     debug.add_argument('-v', '--verbose', action='count', default=0,
                        help="enable verbose logging")
     shared = {'formatter_class': HelpFormatter, 'parents': [debug]}
     parser = argparse.ArgumentParser(prog=GUI, description=__doc__, **shared)
-    # Hidden argument to override the root sharing directory path
+
+    # Build main parser
     parser.add_argument('-j', '--project', metavar="PATH",
                         help="path to the root of the project")
 
@@ -47,14 +51,14 @@ def main(args=None):
 
     # Run the program
     try:
-        success = _run(args, os.getcwd(), parser.error)
+        success = run(args, os.getcwd(), parser.error)
     except KeyboardInterrupt:
-        logging.debug("program interrupted")
+        log.debug("program interrupted")
         success = False
     if success:
-        logging.debug("program exited")
+        log.debug("program exited")
     else:
-        logging.debug("program exited with error")
+        log.debug("program exited with error")
         sys.exit(1)
 
 
@@ -76,7 +80,7 @@ def _configure_logging(verbosity=0):
     logging.root.handlers[0].setFormatter(formatter)
 
 
-def _run(args, cwd, error):
+def run(args, cwd, error):
     """Start the GUI.
 
     :param args: Namespace of CLI arguments (from this module or the CLI)
@@ -103,7 +107,6 @@ def _run(args, cwd, error):
         return True
 
 
-# TODO: cleanup: remove this logging decorator when no longer used
 def _log(func):  # pragma: no cover (manual test)
     """Decorator for methods that should log calls."""
     @functools.wraps(func)
@@ -114,12 +117,12 @@ def _log(func):  # pragma: no cover (manual test)
                                           for k, v in kwargs.items()))
         msg = "{}: {}".format(func.__name__, sargs.strip(", "))
         if not self.ignore:
-            logging.critical(msg.strip())
+            log.critical(msg.strip())
         return func(self, *args, **kwargs)
     return wrapped
 
 
-class Listbox2(tk.Listbox):  # pragma: no cover (manual test), pylint: disable=R0901,R0904
+class Listbox2(tk.Listbox):  # pragma: no cover (manual test), pylint: disable=R0901
 
     """Listbox class with automatic width adjustment."""
 
@@ -138,7 +141,7 @@ class Listbox2(tk.Listbox):  # pragma: no cover (manual test), pylint: disable=R
             self.config(width=width + shift)
 
 
-class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable=R0901,R0902,R0904
+class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable=R0901,R0902
 
     """Graphical application for Doorstop."""
 
@@ -202,7 +205,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
         width_outline = 20
         width_text = 40
         width_code = 30
-        width_id = 10
+        width_uid = 10
         height_text = 10
         height_ext = 5
         height_code = 3
@@ -346,9 +349,9 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
             ttk.Label(frame, text="Properties:").grid(row=2, column=0, sticky=tk.W, **kw_gp)
             ttk.Label(frame, text="Links:").grid(row=2, column=1, columnspan=2, sticky=tk.W, **kw_gp)
             ttk.Checkbutton(frame, text="Active", variable=self.intvar_active).grid(row=3, column=0, sticky=tk.W, **kw_gp)
-            self.listbox_links = tk.Listbox(frame, width=width_id, height=6)
+            self.listbox_links = tk.Listbox(frame, width=width_uid, height=6)
             self.listbox_links.grid(row=3, column=1, rowspan=4, **kw_gsp)
-            ttk.Entry(frame, width=width_id, textvariable=self.stringvar_link).grid(row=3, column=2, sticky=tk.EW + tk.N, **kw_gp)
+            ttk.Entry(frame, width=width_uid, textvariable=self.stringvar_link).grid(row=3, column=2, sticky=tk.EW + tk.N, **kw_gp)
             ttk.Checkbutton(frame, text="Derived", variable=self.intvar_derived).grid(row=4, column=0, sticky=tk.W, **kw_gp)
             ttk.Button(frame, text="<< Link Item", command=self.link).grid(row=4, column=2, **kw_gp)
             ttk.Checkbutton(frame, text="Normative", variable=self.intvar_normative).grid(row=5, column=0, sticky=tk.W, **kw_gp)
@@ -400,7 +403,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
             try:
                 path = vcs.find_root(self.cwd)
             except DoorstopError as exc:
-                logging.error(exc)
+                log.error(exc)
             else:
                 self.stringvar_project.set(path)
 
@@ -408,7 +411,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
     def browse(self):
         """Browse for the root of a project."""
         path = filedialog.askdirectory()
-        logging.debug("path: {}".format(path))
+        log.debug("path: {}".format(path))
         if path:
             self.stringvar_project.set(path)
 
@@ -416,7 +419,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
         """Display the currently selected tree."""
         # Set the current tree
         self.tree = builder.build(root=self.stringvar_project.get())
-        logging.info("displaying tree...")
+        log.info("displaying tree...")
 
         # Display the documents in the tree
         values = ["{} ({})".format(document.prefix, document.relpath)
@@ -431,7 +434,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
         # Set the current document
         index = self.combobox_documents.current()
         self.document = list(self.tree)[index]
-        logging.info("displaying document {}...".format(self.document))
+        log.info("displaying document {}...".format(self.document))
 
         # Display the items in the document
         self.listbox_outline.delete(0, tk.END)
@@ -441,29 +444,29 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
             # Add the item to the document outline
             indent = '  ' * (item.depth - 1)
             level = '.'.join(str(l) for l in item.level)
-            value = "{s}{l} {i}".format(s=indent, l=level, i=item.id)
+            value = "{s}{l} {u}".format(s=indent, l=level, u=item.uid)
             self.listbox_outline.insert(tk.END, value)
 
             # Add the item to the document text
-            value = "{t} [{i}]\n\n".format(t=item.text or item.ref or '???',
-                                           i=item.id)
+            value = "{t} [{u}]\n\n".format(t=item.text or item.ref or '???',
+                                           u=item.uid)
             self.text_items.insert('end', value)
         self.listbox_outline.autowidth()
 
         # Select the first item
         self.listbox_outline.selection_set(self.index or 0)
-        identifier = self.listbox_outline.selection_get()
-        self.stringvar_item.set(identifier)  # manual call
+        uid = self.listbox_outline.selection_get()
+        self.stringvar_item.set(uid)  # manual call
 
     def display_item(self, *_):
         """Display the currently selected item."""
         self.ignore = True
 
         # Set the current item
-        identifier = self.stringvar_item.get().rsplit(' ', 1)[-1]
-        self.item = self.tree.find_item(identifier)
+        uid = self.stringvar_item.get().rsplit(' ', 1)[-1]
+        self.item = self.tree.find_item(uid)
         self.index = self.listbox_outline.curselection()[0]
-        logging.info("displaying item {}...".format(self.item))
+        log.info("displaying item {}...".format(self.item))
 
         # Display the item's text
         self.text_item.replace('1.0', 'end', self.item.text)
@@ -477,8 +480,8 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
 
         # Display the item's links
         self.listbox_links.delete(0, tk.END)
-        for identifier in self.item.links:
-            self.listbox_links.insert(tk.END, identifier)
+        for uid in self.item.links:
+            self.listbox_links.insert(tk.END, uid)
         self.stringvar_link.set('')
 
         # Display the item's external reference
@@ -491,25 +494,25 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
 
         # Display the items this item links to
         self.text_parents.delete('1.0', 'end')
-        for identifier in self.item.links:
+        for uid in self.item.links:
             try:
-                item = self.tree.find_item(identifier)
+                item = self.tree.find_item(uid)
             except DoorstopError:
                 text = "???"
             else:
                 text = item.text or item.ref or '???'
-                identifier = item.id
-            chars = "{t} [{i}]\n\n".format(t=text, i=identifier)
+                uid = item.uid
+            chars = "{t} [{u}]\n\n".format(t=text, u=uid)
             self.text_parents.insert('end', chars)
 
         # Display the items this item has links from
         self.text_children.delete('1.0', 'end')
         identifiers = self.item.find_child_links()
-        for identifier in identifiers:
-            item = self.tree.find_item(identifier)
+        for uid in identifiers:
+            item = self.tree.find_item(uid)
             text = item.text or item.ref or '???'
-            identifier = item.id
-            chars = "{t} [{i}]\n\n".format(t=text, i=identifier)
+            uid = item.uid
+            chars = "{t} [{u}]\n\n".format(t=text, u=uid)
             self.text_children.insert('end', chars)
 
         self.ignore = False
@@ -520,7 +523,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
         self.ignore = True
 
         name = self.stringvar_extendedkey.get()
-        logging.debug("displaying extended attribute '{}'...".format(name))
+        log.debug("displaying extended attribute '{}'...".format(name))
         self.text_extendedvalue.replace('1.0', 'end', self.item.get(name, ''))
 
         self.ignore = False
@@ -531,7 +534,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
             return
 
         # Update the current item
-        logging.info("updating {}...".format(self.item))
+        log.info("updating {}...".format(self.item))
         self.item.auto = False
         self.item.text = self.stringvar_text.get()
         self.item.active = self.intvar_active.get()
@@ -561,7 +564,7 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
         """Increment the current item's level."""
 
     @_log
-    def up(self):  # pylint: disable=C0103
+    def up(self):
         """Decrement the current item's level."""
 
     @_log
@@ -579,9 +582,9 @@ class Application(ttk.Frame):  # pragma: no cover (manual test), pylint: disable
     def link(self):
         """Add the specified link to the current item."""
         # Add the specified link to the list
-        identifier = self.stringvar_link.get()
-        if identifier:
-            self.listbox_links.insert(tk.END, identifier)
+        uid = self.stringvar_link.get()
+        if uid:
+            self.listbox_links.insert(tk.END, uid)
             self.stringvar_link.set('')
 
             # Update the current item
